import gradio as gr
from gradio import utils


class TestChatbot:
    def test_component_functions(self):
        """
        Postprocess, get_config
        """
        chatbot = gr.Chatbot()
        assert chatbot.postprocess(  # type: ignore
            [  # type: ignore
                {"role": "user", "content": "You are **cool**\nand fun"},
                {"role": "assistant", "content": "so are *you*"},
            ]
        ).model_dump() == [
            {
                "role": "user",
                "content": [{"type": "text", "text": "You are **cool**\nand fun"}],
                "metadata": None,
                "options": None,
            },
            {
                "role": "assistant",
                "content": [{"type": "text", "text": "so are *you*"}],
                "metadata": None,
                "options": None,
            },
        ]

        multimodal_msg = [
            {"role": "user", "content": {"path": "test/test_files/video_sample.mp4"}},
            {"role": "assistant", "content": "cool video"},
            {"role": "user", "content": {"path": "test/test_files/audio_sample.wav"}},
            {"role": "assistant", "content": "cool audio"},
            {
                "role": "user",
                "content": {"path": "test/test_files/bus.png", "alt_text": "A bus"},
            },
            {"role": "assistant", "content": "cool pic"},
            {"role": "user", "content": {"path": "test/test_files/video_sample.mp4"}},
            {"role": "assistant", "content": "cool video"},
            {"role": "user", "content": {"path": "test/test_files/audio_sample.wav"}},
            {"role": "assistant", "content": "cool audio"},
            {
                "role": "user",
                "content": {
                    "file": {"path": "test/test_files/bus.png", "alt_text": "A bus"}
                },
            },
            {"role": "assistant", "content": "cool pic"},
        ]
        postprocessed_multimodal_msg = chatbot.postprocess(multimodal_msg).model_dump()  # type: ignore
        for msg in postprocessed_multimodal_msg:
            if msg["role"] == "user":
                assert "file" in msg["content"][0]
                assert msg["content"][0]["file"]["path"].split(".")[-1] in {
                    "mp4",
                    "wav",
                    "png",
                }
                if msg["content"][0]["alt_text"]:
                    assert msg["content"][0]["alt_text"] == "A bus"
            elif msg["role"] == "assistant":
                assert msg["content"][0]["text"] in {
                    "cool video",
                    "cool audio",
                    "cool pic",
                }
            else:
                raise ValueError(f"Unexpected role: {msg['role']}")
        assert chatbot.get_config() == {
            "value": [],
            "label": None,
            "show_label": True,
            "name": "chatbot",
            "visible": True,
            "elem_id": None,
            "elem_classes": [],
            "editable": None,
            "container": True,
            "min_width": 160,
            "group_consecutive_messages": True,
            "scale": None,
            "placeholder": None,
            "height": 400,
            "feedback_options": ["Like", "Dislike"],
            "feedback_value": None,
            "resizable": False,
            "max_height": None,
            "min_height": None,
            "autoscroll": True,
            "proxy_url": None,
            "_selectable": False,
            "_retryable": False,
            "_undoable": False,
            "allow_file_downloads": True,
            "key": None,
            "preserved_by_key": ["value"],
            "latex_delimiters": [{"display": True, "left": "$$", "right": "$$"}],
            "likeable": False,
            "rtl": False,
            "buttons": ["share", "copy", "copy_all"],
            "avatar_images": [None, None],
            "sanitize_html": True,
            "render_markdown": True,
            "line_breaks": True,
            "layout": None,
            "allow_tags": True,
            "examples": None,
            "watermark": None,
            "reasoning_tags": None,
            "like_user_message": False,
        }

    def test_avatar_images_are_moved_to_cache(self):
        chatbot = gr.Chatbot(avatar_images=("test/test_files/bus.png", None))
        assert chatbot.avatar_images[0]
        assert utils.is_in_or_equal(
            chatbot.avatar_images[0]["path"], chatbot.GRADIO_CACHE
        )
        assert chatbot.avatar_images[1] is None

    def test_reasoning_tags_single_block(self):
        """Test reasoning_tags with a single thinking block"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>Let me think about this.</thinking>\nHere is my response.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 2
        assert result[0]["content"][0]["text"] == "Let me think about this."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[1]["content"][0]["text"] == "Here is my response."
        assert result[1]["metadata"] is None

    def test_reasoning_tags_multiple_blocks(self):
        """Test reasoning_tags with multiple thinking blocks"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>First thought.</thinking>\nFirst response.\n<thinking>Second thought.</thinking>\nSecond response.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 4
        assert result[0]["content"][0]["text"] == "First thought."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[1]["content"][0]["text"] == "First response."
        assert result[1]["metadata"] is None
        assert result[2]["content"][0]["text"] == "Second thought."
        assert result[2]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[3]["content"][0]["text"] == "Second response."
        assert result[3]["metadata"] is None

    def test_reasoning_tags_only_thinking(self):
        """Test reasoning_tags with only thinking content, no prose"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>Only thinking here.</thinking>",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 1
        assert result[0]["content"][0]["text"] == "Only thinking here."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}

    def test_reasoning_tags_multiple_tag_types(self):
        """Test reasoning_tags with multiple tag types like <reasoning>"""
        chatbot = gr.Chatbot(
            reasoning_tags=[
                ("<thinking>", "</thinking>"),
                ("<reasoning>", "</reasoning>"),
            ]
        )
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>Thinking block.</thinking>\nFirst response.\n<reasoning>Reasoning block.</reasoning>\nSecond response.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 4
        assert result[0]["content"][0]["text"] == "Thinking block."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[1]["content"][0]["text"] == "First response."
        assert result[1]["metadata"] is None
        assert result[2]["content"][0]["text"] == "Reasoning block."
        assert result[2]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[3]["content"][0]["text"] == "Second response."
        assert result[3]["metadata"] is None

    def test_reasoning_tags_no_thinking_tags(self):
        """Test reasoning_tags when no thinking tags are present"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Just a regular response with no thinking.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 1
        assert (
            result[0]["content"][0]["text"]
            == "Just a regular response with no thinking."
        )
        assert result[0]["metadata"] is None

    def test_reasoning_tags_disabled(self):
        """Test that reasoning_tags=None doesn't extract anything"""
        chatbot = gr.Chatbot(reasoning_tags=None)
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>This should not be extracted.</thinking>\nRegular response.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 1
        assert (
            result[0]["content"][0]["text"]
            == "<thinking>This should not be extracted.</thinking>\nRegular response."
        )
        assert result[0]["metadata"] is None

    def test_reasoning_tags_preserves_order(self):
        """Test that reasoning_tags preserves the order of thinking and prose"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Intro.\n<thinking>Think 1.</thinking>\nMiddle 1.\n<thinking>Think 2.</thinking>\nMiddle 2.\n<thinking>Think 3.</thinking>\nConclusion.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 7
        assert result[0]["content"][0]["text"] == "Intro."
        assert result[0]["metadata"] is None
        assert result[1]["content"][0]["text"] == "Think 1."
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "Middle 1."
        assert result[2]["metadata"] is None
        assert result[3]["content"][0]["text"] == "Think 2."
        assert result[3]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[4]["content"][0]["text"] == "Middle 2."
        assert result[4]["metadata"] is None
        assert result[5]["content"][0]["text"] == "Think 3."
        assert result[5]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[6]["content"][0]["text"] == "Conclusion."
        assert result[6]["metadata"] is None

    def test_reasoning_tags_consecutive_thinking_blocks(self):
        """Test consecutive thinking blocks with no prose between them"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>Think 1.</thinking><thinking>Think 2.</thinking>Prose.<thinking>Think 3.</thinking><thinking>Think 4.</thinking>",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 5
        assert result[0]["content"][0]["text"] == "Think 1."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[1]["content"][0]["text"] == "Think 2."
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "Prose."
        assert result[2]["metadata"] is None
        assert result[3]["content"][0]["text"] == "Think 3."
        assert result[3]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[4]["content"][0]["text"] == "Think 4."
        assert result[4]["metadata"] == {"title": "Reasoning", "status": "done"}

    def test_reasoning_tags_consecutive_prose_sections(self):
        """Test that consecutive prose sections get merged into one message"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Prose 1. Prose 2.<thinking>Think 1.</thinking>Prose 3. Prose 4.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 3
        assert result[0]["content"][0]["text"] == "Prose 1. Prose 2."
        assert result[0]["metadata"] is None
        assert result[1]["content"][0]["text"] == "Think 1."
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "Prose 3. Prose 4."
        assert result[2]["metadata"] is None

    def test_reasoning_tags_prose_before_thinking(self):
        """Test prose at the start followed by thinking blocks"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Starting with prose.<thinking>Then thinking.</thinking>Then more prose.",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 3
        assert result[0]["content"][0]["text"] == "Starting with prose."
        assert result[0]["metadata"] is None
        assert result[1]["content"][0]["text"] == "Then thinking."
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "Then more prose."
        assert result[2]["metadata"] is None

    def test_reasoning_tags_sets_pending_correctly(self):
        """Test prose at the start followed by thinking blocks"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Starting with prose.<thinking>Then thinking",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 2
        assert result[0]["content"][0]["text"] == "Starting with prose."
        assert result[0]["metadata"] is None
        assert result[1]["content"][0]["text"] == "Then thinking"
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "pending"}

    def test_reasoning_tags_sets_pending_correctly_multiple_thinking(self):
        """Test prose at the start followed by thinking blocks"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "Starting with prose.<thinking>Then thinking</thinking><thinking>More thinking",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 3
        assert result[0]["content"][0]["text"] == "Starting with prose."
        assert result[0]["metadata"] is None
        assert result[1]["content"][0]["text"] == "Then thinking"
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "More thinking"
        assert result[2]["metadata"] == {"title": "Reasoning", "status": "pending"}

    def test_reasoning_tags_multiple_thinking_only(self):
        """Test multiple consecutive thinking blocks with no prose at all"""
        chatbot = gr.Chatbot(reasoning_tags=[("<thinking>", "</thinking>")])
        messages = [
            {
                "role": "assistant",
                "content": "<thinking>Think 1.</thinking><thinking>Think 2.</thinking><thinking>Think 3.</thinking>",
            }
        ]
        result = chatbot.postprocess(messages).model_dump()  # type: ignore

        assert len(result) == 3
        assert result[0]["content"][0]["text"] == "Think 1."
        assert result[0]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[1]["content"][0]["text"] == "Think 2."
        assert result[1]["metadata"] == {"title": "Reasoning", "status": "done"}
        assert result[2]["content"][0]["text"] == "Think 3."
        assert result[2]["metadata"] == {"title": "Reasoning", "status": "done"}



"""gr.DownloadButton() component."""

from __future__ import annotations

import tempfile
from collections.abc import Callable, Sequence
from pathlib import Path
from typing import TYPE_CHECKING, Literal

from gradio_client import handle_file
from gradio_client.documentation import document

from gradio.components.base import Component
from gradio.data_classes import FileData
from gradio.events import Events

if TYPE_CHECKING:
    from gradio.components import Timer


@document()
class DownloadButton(Component):
    """
    Creates a button, that when clicked, allows a user to download a single file of arbitrary type.

    Demos: upload_and_download
    """

    EVENTS = [Events.click]

    def __init__(
        self,
        label: str = "Download",
        value: str | Path | Callable | None = None,
        *,
        every: Timer | float | None = None,
        inputs: Component | Sequence[Component] | set[Component] | None = None,
        variant: Literal["primary", "secondary", "stop"] = "secondary",
        visible: bool | Literal["hidden"] = True,
        size: Literal["sm", "md", "lg"] = "lg",
        icon: str | None = None,
        scale: int | None = None,
        min_width: int | None = None,
        interactive: bool = True,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
        preserved_by_key: list[str] | str | None = "value",
    ):
        """
        Parameters:
            label: Text to display on the button. Defaults to "Download".
            value: A str or pathlib.Path filepath or URL to download, or a Callable that returns a str or pathlib.Path filepath or URL to download.
            every: Continously calls `value` to recalculate it if `value` is a function (has no effect otherwise). Can provide a Timer whose tick resets `value`, or a float that provides the regular interval for the reset Timer.
            inputs: Components that are used as inputs to calculate `value` if `value` is a function (has no effect otherwise). `value` is recalculated any time the inputs change.
            variant: 'primary' for main call-to-action, 'secondary' for a more subdued style, 'stop' for a stop button.
            visible: If False, component will be hidden. If "hidden", component will be visually hidden and not take up space in the layout but still exist in the DOM
            size: size of the button. Can be "sm", "md", or "lg".
            icon: URL or path to the icon file to display within the button. If None, no icon will be displayed.
            scale: relative size compared to adjacent Components. For example if Components A and B are in a Row, and A has scale=2, and B has scale=1, A will be twice as wide as B. Should be an integer. scale applies in Rows, and to top-level Components in Blocks where fill_height=True.
            min_width: minimum pixel width, will wrap if not sufficient screen space to satisfy this value. If a certain scale value results in this Component being narrower than min_width, the min_width parameter will be respected first.
            interactive: If False, the UploadButton will be in a disabled state.
            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
            elem_classes: An optional list of strings that are assigned as the classes of this component in the HTML DOM. Can be used for targeting CSS styles.
            render: If False, component will not render be rendered in the Blocks context. Should be used if the intention is to assign event listeners now but render the component later.
            key: in a gr.render, Components with the same key across re-renders are treated as the same component, not a new component. Properties set in 'preserved_by_key' are not reset across a re-render.
            preserved_by_key: A list of parameters from this component's constructor. Inside a gr.render() function, if a component is re-rendered with the same key, these (and only these) parameters will be preserved in the UI (if they have been changed by the user or an event listener) instead of re-rendered based on the values provided during constructor.
        """
        self.data_model = FileData
        self.size = size
        self.label = label
        self.variant = variant
        super().__init__(
            label=label,
            every=every,
            inputs=inputs,
            visible=visible,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render,
            key=key,
            preserved_by_key=preserved_by_key,
            value=value,
            scale=scale,
            min_width=min_width,
            interactive=interactive,
        )
        self.icon = self.serve_static_file(icon)

    def preprocess(self, payload: FileData | None) -> str | None:
        """
        Parameters:
            payload: File information as a FileData object,
        Returns:
            (Rarely used) passes the file as a `str` into the function.
        """
        if payload is None:
            return None
        file_name = payload.path
        file = tempfile.NamedTemporaryFile(delete=False, dir=self.GRADIO_CACHE)
        file.name = file_name
        return file_name

    def postprocess(self, value: str | Path | None) -> FileData | None:
        """
        Parameters:
            value: Expects a `str` or `pathlib.Path` filepath
        Returns:
            File information as a FileData object
        """
        if value is None:
            return None
        return FileData(path=str(value), orig_name=Path(value).name)

    def example_payload(self) -> dict:
        return handle_file(
            "https://github.com/gradio-app/gradio/raw/main/test/test_files/sample_file.pdf"
        )

    def example_value(self) -> str:
        return "https://github.com/gradio-app/gradio/raw/main/test/test_files/sample_file.pdf"

    @property
    def skip_api(self):
        return False



from __future__ import annotations

from collections.abc import Iterable

from gradio.themes.base import Base
from gradio.themes.utils import colors, fonts, sizes


class Origin(Base):
    def __init__(
        self,
        *,
        primary_hue: colors.Color | str = colors.orange,
        secondary_hue: colors.Color | str = colors.blue,
        neutral_hue: colors.Color | str = colors.gray,
        spacing_size: sizes.Size | str = sizes.spacing_md,
        radius_size: sizes.Size | str = sizes.radius_md,
        text_size: sizes.Size | str = sizes.text_md,
        font: fonts.Font | str | Iterable[fonts.Font | str] = (
            fonts.GoogleFont("Source Sans Pro"),
            "ui-sans-serif",
            "system-ui",
            "sans-serif",
        ),
        font_mono: fonts.Font | str | Iterable[fonts.Font | str] = (
            fonts.LocalFont("IBM Plex Mono"),
            "ui-monospace",
            "Consolas",
            "monospace",
        ),
    ):
        super().__init__(
            primary_hue=primary_hue,
            secondary_hue=secondary_hue,
            neutral_hue=neutral_hue,
            spacing_size=spacing_size,
            radius_size=radius_size,
            text_size=text_size,
            font=font,
            font_mono=font_mono,
        )
        self.name = "origin"
        super().set(
            # Colors
            input_background_fill_dark="*neutral_800",
            error_background_fill=colors.red.c50,
            error_background_fill_dark="*neutral_900",
            error_border_color=colors.red.c700,
            error_border_color_dark=colors.red.c500,
            error_icon_color=colors.red.c700,
            error_icon_color_dark=colors.red.c500,
            # Transition
            button_transition="none",
            # Shadows
            button_primary_shadow="*shadow_drop",
            button_primary_shadow_hover="*shadow_drop_lg",
            button_primary_shadow_active="*shadow_inset",
            button_secondary_shadow="*shadow_drop",
            button_secondary_shadow_hover="*shadow_drop_lg",
            button_secondary_shadow_active="*shadow_inset",
            input_shadow="0 0 0 *shadow_spread transparent, *shadow_inset",
            input_shadow_focus="0 0 0 *shadow_spread *secondary_50, *shadow_inset",
            input_shadow_focus_dark="0 0 0 *shadow_spread *neutral_700, *shadow_inset",
            checkbox_label_shadow="*shadow_drop",
            block_shadow="*shadow_drop",
            form_gap_width="1px",
            # Button borders
            button_border_width="1px",
            button_border_width_dark="1px",
            input_border_width="1px",
            input_background_fill="white",
            # Gradients
            stat_background_fill="linear-gradient(to right, *primary_400, *primary_200)",
            stat_background_fill_dark="linear-gradient(to right, *primary_400, *primary_600)",
            checkbox_label_background_fill="linear-gradient(to top, *neutral_50, white)",
            checkbox_label_background_fill_dark="linear-gradient(to top, *neutral_900, *neutral_800)",
            checkbox_label_background_fill_hover="linear-gradient(to top, *neutral_100, white)",
            checkbox_label_background_fill_hover_dark="linear-gradient(to top, *neutral_900, *neutral_800)",
            # Primary Button
            button_primary_background_fill="linear-gradient(to bottom right, *primary_100, *primary_300)",
            button_primary_background_fill_dark="linear-gradient(to bottom right, *primary_500, *primary_600)",
            button_primary_background_fill_hover="linear-gradient(to bottom right, *primary_100, *primary_200)",
            button_primary_background_fill_hover_dark="linear-gradient(to bottom right, *primary_500, *primary_500)",
            button_primary_border_color="*primary_200",
            button_primary_border_color_dark="*primary_500",
            button_primary_border_color_hover="*button_primary_border_color",
            button_primary_border_color_hover_dark="*primary_500",
            button_primary_text_color="*primary_600",
            button_primary_text_color_dark="white",
            # Secondary Button
            button_secondary_background_fill="linear-gradient(to bottom right, *neutral_100, *neutral_200)",
            button_secondary_background_fill_dark="linear-gradient(to bottom right, *neutral_600, *neutral_700)",
            button_secondary_background_fill_hover="linear-gradient(to bottom right, *neutral_100, *neutral_100)",
            button_secondary_background_fill_hover_dark="linear-gradient(to bottom right, *neutral_600, *neutral_600)",
            button_secondary_border_color="*neutral_200",
            button_secondary_border_color_dark="*neutral_600",
            button_secondary_border_color_hover="*neutral_200",
            button_secondary_border_color_hover_dark="*neutral_600",
            button_secondary_text_color="*neutral_800",
            button_secondary_text_color_dark="white",
            # Cancel Button
            button_cancel_background_fill=f"linear-gradient(to bottom right, {colors.red.c100}, {colors.red.c200})",
            button_cancel_background_fill_dark=f"linear-gradient(to bottom right, {colors.red.c600}, {colors.red.c700})",
            button_cancel_background_fill_hover=f"linear-gradient(to bottom right, {colors.red.c100}, {colors.red.c100})",
            button_cancel_background_fill_hover_dark=f"linear-gradient(to bottom right, {colors.red.c600}, {colors.red.c600})",
            button_cancel_border_color=colors.red.c200,
            button_cancel_border_color_dark=colors.red.c600,
            button_cancel_border_color_hover=colors.red.c200,
            button_cancel_border_color_hover_dark=colors.red.c600,
            button_cancel_text_color=colors.red.c600,
            button_cancel_text_color_dark="white",
            # Other
            border_color_accent_subdued="*primary_200",
            button_transform_hover="none",
        )



import unittest
from unittest.mock import MagicMock

import pytest
import transformers
from transformers import (
    AudioClassificationPipeline,
    AutomaticSpeechRecognitionPipeline,
    DocumentQuestionAnsweringPipeline,
    FeatureExtractionPipeline,
    FillMaskPipeline,
    ImageClassificationPipeline,
    ObjectDetectionPipeline,
    QuestionAnsweringPipeline,
    TextClassificationPipeline,
    TextGenerationPipeline,
    VisualQuestionAnsweringPipeline,
    ZeroShotClassificationPipeline,
)

import gradio as gr
from gradio.pipelines_utils import (
    handle_transformers_pipeline,
)


@pytest.mark.flaky
def test_interface_in_blocks():
    pipe1 = transformers.pipeline(model="deepset/roberta-base-squad2")  # type: ignore
    pipe2 = transformers.pipeline(model="deepset/roberta-base-squad2")  # type: ignore
    with gr.Blocks() as demo:
        with gr.Tab("Image Inference"):
            gr.Interface.from_pipeline(pipe1)
        with gr.Tab("Image Inference"):
            gr.Interface.from_pipeline(pipe2)
    demo.launch(prevent_thread_lock=True)
    demo.close()


@pytest.mark.flaky
def test_transformers_load_from_pipeline():
    from transformers import pipeline

    pipe = pipeline(model="deepset/roberta-base-squad2")  # type: ignore
    io = gr.Interface.from_pipeline(pipe)
    assert io.input_components[0].label == "Context"  # type: ignore
    assert io.input_components[1].label == "Question"  # type: ignore
    assert io.output_components[0].label == "Answer"  # type: ignore
    assert io.output_components[1].label == "Score"  # type: ignore


class TestHandleTransformersPipelines(unittest.TestCase):
    def test_audio_classification_pipeline(self):
        pipe = MagicMock(spec=AudioClassificationPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Class"

    def test_automatic_speech_recognition_pipeline(self):
        pipe = MagicMock(spec=AutomaticSpeechRecognitionPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Output"

    def test_object_detection_pipeline(self):
        pipe = MagicMock(spec=ObjectDetectionPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input Image"
        assert pipeline_info["outputs"].label == "Objects Detected"

    def test_feature_extraction_pipeline(self):
        pipe = MagicMock(spec=FeatureExtractionPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Output"

    def test_fill_mask_pipeline(self):
        pipe = MagicMock(spec=FillMaskPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Classification"

    def test_image_classification_pipeline(self):
        pipe = MagicMock(spec=ImageClassificationPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input Image"
        assert pipeline_info["outputs"].label == "Classification"

    def test_question_answering_pipeline(self):
        pipe = MagicMock(spec=QuestionAnsweringPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"][0].label == "Context"
        assert pipeline_info["inputs"][1].label == "Question"
        assert pipeline_info["outputs"][0].label == "Answer"
        assert pipeline_info["outputs"][1].label == "Score"

    def test_text_classification_pipeline(self):
        pipe = MagicMock(spec=TextClassificationPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Classification"

    def test_text_generation_pipeline(self):
        pipe = MagicMock(spec=TextGenerationPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"].label == "Input"
        assert pipeline_info["outputs"].label == "Output"

    def test_zero_shot_classification_pipeline(self):
        pipe = MagicMock(spec=ZeroShotClassificationPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"][0].label == "Input"
        assert (
            pipeline_info["inputs"][1].label == "Possible class names (comma-separated)"
        )
        assert pipeline_info["inputs"][2].label == "Allow multiple true classes"
        assert pipeline_info["outputs"].label == "Classification"

    def test_document_question_answering_pipeline(self):
        pipe = MagicMock(spec=DocumentQuestionAnsweringPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"][0].label == "Input Document"
        assert pipeline_info["inputs"][1].label == "Question"
        assert pipeline_info["outputs"].label == "Label"

    def test_visual_question_answering_pipeline(self):
        pipe = MagicMock(spec=VisualQuestionAnsweringPipeline)
        pipeline_info = handle_transformers_pipeline(pipe)
        assert pipeline_info is not None
        assert pipeline_info["inputs"][0].label == "Input Image"
        assert pipeline_info["inputs"][1].label == "Question"
        assert pipeline_info["outputs"].label == "Score"

    def test_unsupported_pipeline(self):
        pipe = MagicMock()
        with self.assertRaises(ValueError):
            handle_transformers_pipeline(pipe)



from gradio.image_utils import extract_svg_content


def test_extract_svg_content_local_file():
    svg_path = "test/test_files/file_icon.svg"
    svg_content = extract_svg_content(svg_path)
    assert (
        svg_content
        == '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round" class="feather feather-file"><path d="M13 2H6a2 2 0 0 0-2 2v16a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V9z"></path><polyline points="13 2 13 9 20 9"></polyline></svg>'
    )


def test_extract_svg_content_from_url(monkeypatch):
    class MockResponse:
        def __init__(self):
            self.text = "<svg>mock svg content</svg>"

        def raise_for_status(self):
            pass

    def mock_get(*args, **kwargs):
        return MockResponse()

    monkeypatch.setattr("httpx.get", mock_get)
    svg_content = extract_svg_content("https://example.com/test.svg")
    assert svg_content == "<svg>mock svg content</svg>"



from __future__ import annotations

import re
import shutil
import tempfile
from pathlib import Path
from typing import Annotated

import semantic_version
from huggingface_hub import HfApi
from packaging.version import Version as PEP440Version
from rich import print
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm, Prompt
from tomlkit import parse
from typer import Argument, Option

from gradio.analytics import custom_component_analytics

colors = ["red", "yellow", "green", "blue", "indigo", "purple", "pink", "gray"]

PYPI_REGISTER_URL = "https://pypi.org/account/register/"


def _ignore(_src, names):
    ignored = []
    for n in names:
        if "__pycache__" in n or n.startswith("dist") or n.startswith("node_modules"):
            ignored.append(n)
    return ignored


def _get_version_from_file(dist_file: Path) -> str | None:
    match = re.search(r"-(\d+\.\d+\.\d+[a-zA-Z]*\d*)-", dist_file.name)
    if match:
        return match.group(1)
    return None


def _parse_version(
    version_string: str,
) -> semantic_version.Version | PEP440Version | None:
    """Parse a version string, trying semantic_version first, then PEP440."""
    try:
        return semantic_version.Version(version_string)
    except ValueError:
        try:
            return PEP440Version(version_string)
        except Exception:
            return None


def _get_max_version(distribution_files: list[Path]) -> str | None:
    versions = []
    for p in distribution_files:
        version = _get_version_from_file(p)
        # If anything goes wrong, just return None so we upload all files
        # better safe than sorry
        if version:
            parsed = _parse_version(version)
            if parsed:
                versions.append(parsed)
            else:
                return None
    return str(max(versions)) if versions else None


def _publish(
    dist_dir: Annotated[
        Path,
        Argument(help=f"Path to the wheel directory. Default is {Path('.') / 'dist'}"),
    ] = Path(".") / "dist",
    upload_pypi: Annotated[bool, Option(help="Whether to upload to PyPI.")] = True,
    pypi_username: Annotated[str, Option(help="The username for PyPI.")] = "",
    pypi_password: Annotated[str, Option(help="The password for PyPI.")] = "",
    upload_demo: Annotated[
        bool, Option(help="Whether to upload demo to HuggingFace.")
    ] = True,
    demo_dir: Annotated[Path | None, Option(help="Path to the demo directory.")] = None,
    source_dir: Annotated[
        Path,
        Option(help="Path to the source directory of the custom component."),
    ] = Path("."),
    token: Annotated[
        str | None,
        Option(
            help="HuggingFace token for uploading demo. Can be omitted if already logged in via huggingface cli."
        ),
    ] = None,
    prefer_local: Annotated[
        bool,
        Option(
            help="Install the package from the local wheel in the demo space, even if it exists on PyPi."
        ),
    ] = False,
    upload_source: Annotated[
        bool,
        Option(
            help="Whether to upload the source code of the custom component, to share with the community."
        ),
    ] = False,
    repo_id: Annotated[
        str | None,
        Option(
            help="The repository id to upload the demo to. If not provided, a space will be created with the same name as the package in the HuggingFace account corresponding to the token."
        ),
    ] = None,
):
    custom_component_analytics(
        "publish",
        None,
        upload_demo=upload_demo,
        upload_pypi=upload_pypi,
        upload_source=upload_source,
    )
    console = Console()
    dist_dir = dist_dir.resolve()

    if not dist_dir.exists():
        raise ValueError(
            f"{dist_dir} does not exist. Run `gradio cc build` to create a wheel and source distribution."
        )
    if not dist_dir.is_dir():
        raise ValueError(f"{dist_dir} is not a directory")
    distribution_files = [
        p.resolve() for p in Path(dist_dir).glob("*") if p.suffix in {".whl", ".gz"}
    ]

    def _get_wheel_version(path: Path):
        version_str = str(path.name).split("-")[1]
        parsed = _parse_version(version_str)
        if parsed is None:
            raise ValueError(f"Could not parse version {version_str} from {path.name}")
        return parsed

    wheel_file = max(
        (p for p in distribution_files if p.suffix == ".whl"),
        key=_get_wheel_version,
    )
    if not wheel_file:
        raise ValueError(
            "A wheel file was not found in the distribution directory. "
            "Run `gradio cc build` to create a wheel file."
        )
    config_file = None
    if upload_pypi and (not pypi_username or not pypi_password):
        panel = Panel(
            "It is recommended to upload your component to pypi so that [bold][magenta]anyone[/][/] "
            "can install it with [bold][magenta]pip install[/][/].\n\n"
            f"A PyPi account is needed. If you do not have an account, register account here: [blue]{PYPI_REGISTER_URL}[/]",
        )
        print(panel)
        upload_pypi = Confirm.ask(":snake: Upload to pypi?")
        if upload_pypi and (Path.home() / ".pypirc").exists():
            print(":closed_lock_with_key: Found .pypirc file in home directory.")
            config_file = str(Path.home() / ".pypirc")
        elif upload_pypi:
            print(
                ":light_bulb: If you have Two Factor Authentication enabled, the username is __token__ and your password is your API key."
            )
            pypi_username = Prompt.ask(":laptop_computer: Enter your pypi username")
            pypi_password = Prompt.ask(
                ":closed_lock_with_key: Enter your pypi password", password=True
            )
    if upload_pypi:
        try:
            from twine.commands.upload import upload as twine_upload  # type: ignore
            from twine.exceptions import InvalidDistribution  # type: ignore
            from twine.settings import Settings  # type: ignore
        except (ImportError, ModuleNotFoundError) as e:
            raise ValueError(
                "The twine library must be installed to publish to pypi."
                "Install it with pip, pip install twine."
            ) from e
        if pypi_username and pypi_password:
            twine_settings = Settings(username=pypi_username, password=pypi_password)
        elif config_file:
            twine_settings = Settings(config_file=config_file)
        else:
            raise ValueError(
                "No pypi username or password provided and no ~/.pypirc file found."
            )
        try:
            # do our best to only upload the latest versions
            max_version = _get_max_version(distribution_files)
            twine_files = [
                str(p)
                for p in distribution_files
                if (not max_version or max_version in p.name)
            ]
            print(f"Uploading files: {','.join(twine_files)}")
            try:
                twine_upload(twine_settings, twine_files)
            except InvalidDistribution as e:
                raise ValueError(
                    "Invalid distribution when uploading to pypi. "
                    "Try upgrading 'pkginfo' with python -m pip install pkginfo --upgrade"
                ) from e
        except Exception:
            console.print_exception()
    if upload_demo and not demo_dir:
        panel = Panel(
            "It is recommended you upload a demo of your component to [blue]https://huggingface.co/spaces[/] "
            "so that anyone can try it from their browser."
        )
        print(panel)
        upload_demo = Confirm.ask(":hugging_face: Upload demo?")
        if upload_demo:
            panel = Panel(
                "Please provide the path to the [magenta]demo directory[/] for your custom component.\n\n"
                "This directory should contain [magenta]all the files[/] it needs to run successfully.\n\n"
                "Please make sure the gradio app is in an [magenta]space.py[/] file.\n\n"
                "If you need additional python requirements, add a [magenta]requirements.txt[/] file to this directory."
            )
            print(panel)
            demo_dir_ = Prompt.ask(
                f":roller_coaster: Please enter the path to the demo directory. Leave blank to use: {(Path('.') / 'demo')}"
            )
            demo_dir_ = demo_dir_ or str(Path(".") / "demo")
            demo_dir = Path(demo_dir_).resolve()

    if upload_demo and not upload_source:
        panel = Panel(
            "It is recommended that you share your [magenta]source code[/] so that others can learn from and improve your component."
        )
        print(panel)
        upload_source = Confirm.ask(":books: Would you like to share your source code?")
        if upload_source:
            source_dir_ = (
                Prompt.ask(
                    f":file_folder: Please enter the path to the source directory. Leave blank to use: {source_dir}"
                )
                or source_dir
            )
            source_dir = Path(source_dir_).resolve()

    if upload_demo:
        pyproject_toml_path = source_dir / "pyproject.toml"

        try:
            pyproject_toml = parse(pyproject_toml_path.read_text())
            package_name = pyproject_toml["project"]["name"]  # type: ignore
        except Exception:
            (package_name, _) = wheel_file.name.split("-")[:2]

        if not demo_dir:
            raise ValueError("demo_dir must be set")

        package_name, _ = wheel_file.name.split("-")[:2]
        with tempfile.TemporaryDirectory() as tempdir:
            shutil.copytree(
                str(demo_dir),
                str(tempdir),
                dirs_exist_ok=True,
            )
            shutil.copyfile(
                str(source_dir / ".gitignore"),
                str(Path(tempdir) / ".gitignore"),
            )
            if upload_source:
                shutil.copytree(
                    str(source_dir),
                    str(Path(tempdir) / "src"),
                    dirs_exist_ok=True,
                    ignore=_ignore,
                )

            shutil.copyfile(
                str(source_dir / "README.md"), str(Path(tempdir) / "README.md")
            )

            api = HfApi(token=token)
            repo_url = api.create_repo(
                repo_id=repo_id or package_name,
                repo_type="space",
                exist_ok=True,
                space_sdk="gradio",
            )
            repo_id = repo_url.repo_id
            api.upload_folder(
                repo_id=repo_id,
                folder_path=tempdir,
                repo_type="space",
            )
            if prefer_local:
                api.upload_file(
                    repo_id=repo_id,
                    path_or_fileobj=str(wheel_file),
                    path_in_repo=wheel_file.name,
                    repo_type="space",
                )
            print("\n")
            # Do a factory reboot so that the new dependencies get installed
            api.restart_space(repo_id=repo_id, factory_reboot=True, token=token)
            print(f"Demo uploaded to https://huggingface.co/spaces/{repo_id} !")


def resolve_demo(demo_dir: Path) -> Path:
    _demo_dir = demo_dir.resolve()
    if (_demo_dir / "space.py").exists():
        return _demo_dir / "space.py"
    elif (_demo_dir / "app.py").exists():
        return _demo_dir / "app.py"
    else:
        raise FileNotFoundError(
            f'Could not find "space.py" or "app.py" in "{demo_dir}".'
        )



import libsql_client as libsql
import os
from openai import OpenAI
import numpy as np
from chunking import TextChunker
from tqdm import tqdm
import requests 
import voyageai

# vo = voyageai.Client(api_key=os.getenv("VOYAGE_API_KEY"),)
url = os.getenv("TURSO_DATABASE_URL")
auth_token = os.getenv("TURSO_AUTH_TOKEN")
db_client = libsql.create_client_sync(url, auth_token=auth_token)

# openai_deepinfra = OpenAI(
#     api_key=os.getenv("DEEPINFRA_API_TOKEN"),
#     base_url="https://api.deepinfra.com/v1/openai"
# )


# def embed_and_upload(title, _type, url, contents):
#     documents_embeddings = vo.embed(contents, model="voyage-3", input_type="document").embeddings
    
#     values = [
#         (title, _type, url, content, np.array(embedding, dtype=np.float32))
#         for content, embedding in zip(contents, documents_embeddings)
#     ]
    
#     placeholders = ','.join(['(?, ?, ?, ?, ?)'] * len(contents))
    
#     flattened_values = [item for tup in values for item in tup]
    
#     db_client.execute(
#         f"INSERT INTO EMBEDDINGS (title, type, url, content, embedding) VALUES {placeholders}", 
#         flattened_values
#     )
#     return

# url = "http://localhost:5174/search-api"
# response = requests.get(url)
# data = response.json()

# guides = [d for d in data if d["type"] == "GUIDE"]
# docs = [d for d in data if d["type"] == "DOCS"]

# chunker = TextChunker()

# DOCS_AND_GUIDES_DESCRIPTION_SYSTEM_PROMPT = """
# You are a helpful assistant that summarizes pages in the Gradio website in only one sentence.
# You are given a page that is either a guide or docs. Both will consist of natural language mixed with python code.
# Your summaries will be used for embedding search that points to the page, so please be concise, accurate and include the most important parts. But it can only be one sentence.
# Your sentence should clarify what type of questions the page answers.
# Do not include 'gr.' before the function or class name. Do not ever use backticks or special code formatting in your response. For example write Interface instead of `Interface`.
# """

# def describe_page(content: str):
#     description = openai_deepinfra.chat.completions.create(
#         model="Qwen/Qwen2.5-72B-Instruct",
#         messages=[
#             {"role": "system", "content": DOCS_AND_GUIDES_DESCRIPTION_SYSTEM_PROMPT},
#             {"role": "user", "content": content}
#             ],
#     )

#     description = description.choices[0].message.content

#     return description

# for guide in tqdm(guides[1:]): # ignore weird 
#     description = describe_page(guide["content"])
#     chunks = chunker.chunk_page(guide["title"], guide["slug"], description, guide["type"])
#     try:
#         embed_and_upload(chunks.title, chunks.type, chunks.url, chunks.content)
#     except Exception as e:
#         print(e)
#         db_client.close()
#         1/0

# for page in tqdm(docs):
#     description = describe_page(guide["content"])
#     chunks = chunker.chunk_page(guide["title"], guide["slug"], description, guide["type"])
#     try:
#         embed_and_upload(chunks.title, chunks.type, chunks.url, chunks.content)
#     except Exception as e:
#         print(e)
#         db_client.close()
#         1/0


# demo_descriptions = []
# def get_demo_descriptions():
#     results = db_client.execute(
#         """
#         SELECT 
#             MIN(id) as id,
#             title,
#             type,
#             url,
#             STRING_AGG(content, ' ') as combined_content
#         FROM EMBEDDINGS_LLM_250
#         WHERE type = 'DEMO'
#         GROUP BY title, type, url;
#         """
#     )
#     for result in results:
#         demo_descriptions.append(
#             {
#                 "title": result["title"],
#                 "url": result["url"],
#                 "content": result["combined_content"]
#             }
#         )
#     return

# get_demo_descriptions()

# for demo in tqdm(demo_descriptions):
#     chunks = chunker.chunk_page(demo["title"], demo["url"], demo["content"], "DEMO")
#     try:
#         embed_and_upload(chunks.title, chunks.type, chunks.url, chunks.content)
#     except Exception as e:
#         print(e)
#         db_client.close()
#         1/0

demo_to_reqs = {}
for demo in os.listdir("demo"):
    if os.path.exists(os.path.join("demo", demo, "requirements.txt")):
        with open(os.path.join("demo", demo, "requirements.txt"), "r") as f:
            reqs = f.read()
            reqs = reqs.split("\n")
            demo_to_reqs[demo] = reqs

for title, requirements in tqdm(demo_to_reqs.items()):
    db_client.execute(
        """UPDATE EMBEDDINGS 
        SET requirements = ?
        WHERE type = 'DEMO' AND title = ?
        """, 
        (requirements, title.replace("_", " ").capitalize())
    )
    print(title.replace("_", " ").capitalize())

# demo_to_reqs = {}
# for demo in os.listdir("demo"):
#     if os.path.exists(os.path.join("demo", demo, "requirements.txt")):
#         with open(os.path.join("demo", demo, "requirements.txt"), "r") as f:
#             demo_to_reqs[demo] = f.read()

# for title, requirements in tqdm(demo_to_reqs.items()):
#     db_client.execute(
#         """UPDATE EMBEDDINGS 
#         SET requirements = ?
#         WHERE type = 'DEMO' AND title = ?
#         """, 
#         (requirements, title.replace("_", " ").capitalize())
#     )
#     print(title.replace("_", " ").capitalize())


db_client.close()



from __future__ import annotations

import os
import re
import shutil
import subprocess
from typing import Annotated

import huggingface_hub
from rich import print
from typer import Option

import gradio as gr

repo_directory = os.getcwd()
readme_file = os.path.join(repo_directory, "README.md")
github_action_template = os.path.join(
    os.path.dirname(__file__), "deploy_space_action.yaml"
)


def add_configuration_to_readme(
    title: str | None,
    app_file: str | None,
) -> dict:
    configuration = {}

    dir_name = os.path.basename(repo_directory)
    if title is None:
        title = input(f"Enter Spaces app title [{dir_name}]: ") or dir_name
    formatted_title = format_title(title)
    if formatted_title != title:
        print(f"Formatted to {formatted_title}. ")
    configuration["title"] = formatted_title

    if app_file is None:
        for file in os.listdir(repo_directory):
            file_path = os.path.join(repo_directory, file)
            if not os.path.isfile(file_path) or not file.endswith(".py"):
                continue

            with open(file_path, encoding="utf-8", errors="ignore") as f:
                content = f.read()
                if "import gradio" in content:
                    app_file = file
                    break

        app_file = (
            input(f"Enter Gradio app file {f'[{app_file}]' if app_file else ''}: ")
            or app_file
        )
    if not app_file or not os.path.exists(app_file):
        raise FileNotFoundError("Failed to find Gradio app file.")
    configuration["app_file"] = app_file

    configuration["sdk"] = "gradio"
    configuration["sdk_version"] = gr.__version__
    huggingface_hub.metadata_save(readme_file, configuration)

    configuration["hardware"] = (
        input(
            f"Enter Spaces hardware ({', '.join(hardware.value for hardware in huggingface_hub.SpaceHardware)}) [cpu-basic]: "
        )
        or "cpu-basic"
    )

    secrets = {}
    if input("Any Spaces secrets (y/n) [n]: ") == "y":
        while True:
            secret_name = input("Enter secret name (leave blank to end): ")
            if not secret_name:
                break
            secret_value = input(f"Enter secret value for {secret_name}: ")
            secrets[secret_name] = secret_value
    configuration["secrets"] = secrets

    requirements_file = os.path.join(repo_directory, "requirements.txt")
    if (
        not os.path.exists(requirements_file)
        and input("Create requirements.txt file? (y/n) [n]: ").lower() == "y"
    ):
        while True:
            requirement = input("Enter a dependency (leave blank to end): ")
            if not requirement:
                break
            with open(requirements_file, "a", encoding="utf-8") as f:
                f.write(requirement + "\n")

    if (
        input(
            "Create Github Action to automatically update Space on 'git push'? [n]: "
        ).lower()
        == "y"
    ):
        track_branch = input("Enter branch to track [main]: ") or "main"
        github_action_file = os.path.join(
            repo_directory, ".github/workflows/update_space.yml"
        )
        os.makedirs(os.path.dirname(github_action_file), exist_ok=True)
        with open(github_action_template, encoding="utf-8") as f:
            github_action_content = f.read()
        github_action_content = github_action_content.replace("$branch", track_branch)
        with open(github_action_file, "w", encoding="utf-8") as f:
            f.write(github_action_content)

        print(
            "Github Action created. Add your Hugging Face write token (from https://huggingface.co/settings/tokens) as an Actions Secret named 'hf_token' to your GitHub repository. This can be set in your repository's settings page."
        )

    return configuration


def format_title(title: str):
    title = title.replace(" ", "_")
    title = re.sub(r"[^a-zA-Z0-9\-._]", "", title)
    title = re.sub("-+", "-", title)
    while title.startswith("."):
        title = title[1:]
    return title


def check_gcloud_auth():
    """Check if user is logged in to Google Cloud and has a project selected."""
    try:
        auth_result = subprocess.run(
            ["gcloud", "auth", "list", "--filter=status:ACTIVE"],
            check=True,
            text=True,
            capture_output=True,
        )

        if not auth_result.stdout.strip():
            print("[bold yellow]You are not logged in to Google Cloud.[/bold yellow]")
            print("Running 'gcloud init' to set up authentication...")
            subprocess.run(["gcloud", "init"], check=True)

        project_result = subprocess.run(
            ["gcloud", "config", "get-value", "project"],
            check=True,
            text=True,
            capture_output=True,
        )

        if not project_result.stdout.strip():
            print("[bold yellow]No Google Cloud project is selected.[/bold yellow]")
            project_id = input("Enter your Google Cloud project ID: ")
            if not project_id:
                print("[red]Project ID is required for deployment.[/red]")
                return None
            return project_id

        print(f"[green]✓ Authenticated as: {auth_result.stdout.strip()}[/green]")
        print(f"[green]✓ Project: {project_result.stdout.strip()}[/green]")
        return project_result.stdout.strip()

    except subprocess.CalledProcessError as e:
        print(f"[bold red]Error checking Google Cloud configuration: {e}[/bold red]")
        print("Running 'gcloud init' to set up configuration...")
        try:
            subprocess.run(["gcloud", "init"], check=True)
        except subprocess.CalledProcessError as init_error:
            print(f"[red]Failed to run 'gcloud init': {init_error}[/red]")
            return False
    except FileNotFoundError:
        print(
            "[bold red]gcloud CLI not found. Please install Google Cloud SDK.[/bold red]"
        )
        return False

    return True


def deploy_to_gcloud():
    """Deploy a Gradio app to Google Cloud Run. Always uses app.py as the entry point."""
    if not shutil.which("gcloud"):
        print(
            "[bold red]gcloud CLI is not installed.[/bold red]\n"
            "Please install the Google Cloud SDK from: "
            "[link]https://cloud.google.com/sdk/docs/install[/link]"
        )
        return

    project_id = check_gcloud_auth()
    if not project_id:
        print(
            "[bold red]Google Cloud configuration failed. Please run 'gcloud init' manually.[/bold red]"
        )
        return

    if not os.path.exists("app.py") and not os.path.exists("main.py"):
        print(
            "[bold red]Error:[/bold red] app.py and main.py not found. Google Cloud Run deployment requires app.py or main.py as the entry point."
        )
        return

    requirements_file = "requirements.txt"
    if not os.path.exists(requirements_file):
        if (
            input(
                f"A requirements.txt file is necessary for Google Cloud Run deployment. Create requirements.txt with gradio=={gr.__version__}? (y/n) [y]: "
            ).lower()
            != "n"
        ):
            with open(requirements_file, "w", encoding="utf-8") as f:
                f.write(f"gradio=={gr.__version__}\n")
            print(f"Created requirements.txt with gradio=={gr.__version__}")
        else:
            print("\n[yellow]Deployment cancelled.[/yellow]")
            return
    else:
        with open(requirements_file, encoding="utf-8") as f:
            requirements_content = f.read()

        if "gradio" not in requirements_content and (
            input(
                f"Add gradio=={gr.__version__} to requirements.txt? (y/n) [y]: "
            ).lower()
            != "n"
        ):
            with open(requirements_file, "a", encoding="utf-8") as f:
                f.write(f"gradio=={gr.__version__}\n")
            print(f"Added gradio=={gr.__version__} to requirements.txt")

    print("[bold]Deploying to Google Cloud Run...[/bold]")

    try:
        deploy_command = [
            "gcloud",
            "run",
            "deploy",
            "--source=.",
            "--labels=created-by=gradio",
        ]
        if project_id:
            deploy_command.extend(["--project", project_id])

        subprocess.run(
            deploy_command,
            check=True,
            text=True,
            capture_output=False,
        )
        print("[green]✓ Deployment complete![/green]")
    except subprocess.CalledProcessError as e:
        print(f"[red]Deployment failed: {e}[/red]")
        return
    except KeyboardInterrupt:
        print("\n[yellow]Deployment cancelled.[/yellow]")
        return


def deploy(
    title: Annotated[str | None, Option(help="Spaces app title")] = None,
    app_file: Annotated[
        str | None, Option(help="File containing the Gradio app")
    ] = None,
    provider: Annotated[
        str | None, Option(help="Deployment provider (spaces or gcloud)")
    ] = "spaces",
):
    if provider == "gcloud":
        if app_file and app_file != "app.py":
            print(
                "[yellow]Warning:[/yellow] --app-file is ignored for Google Cloud Run deployment. Using app.py as entry point."
            )
        deploy_to_gcloud()
        return

    if provider != "spaces":
        print(f"[red]Unknown provider: {provider}. Use 'spaces' or 'gcloud'.[/red]")
        return

    if os.getenv("SYSTEM") == "spaces":
        return

    hf_api = huggingface_hub.HfApi()
    whoami = None
    login = False
    try:
        whoami = hf_api.whoami()
        if whoami["auth"]["accessToken"]["role"] != "write":
            login = True
    except OSError:
        login = True
    if login:
        print("Need 'write' access token to create a Spaces repo.")
        huggingface_hub.login(add_to_git_credential=False)
        whoami = hf_api.whoami()

    configuration: None | dict = None
    if os.path.exists(readme_file):
        try:
            configuration = huggingface_hub.metadata_load(readme_file)
        except ValueError:
            pass

    if configuration is None:
        print(
            f"Creating new Spaces Repo in '{repo_directory}'. Collecting metadata, press Enter to accept default value."
        )
        configuration = add_configuration_to_readme(
            title,
            app_file,
        )

    space_id = hf_api.create_repo(
        configuration["title"],
        space_sdk="gradio",
        repo_type="space",
        exist_ok=True,
        space_hardware=configuration.get("hardware"),
    ).repo_id
    hf_api.upload_folder(
        repo_id=space_id,
        repo_type="space",
        folder_path=repo_directory,
    )
    if configuration.get("secrets"):
        for secret_name, secret_value in configuration["secrets"].items():
            hf_api.add_space_secret(space_id, secret_name, secret_value)
    print(f"Space available at https://huggingface.co/spaces/{space_id}")



from pathlib import Path

import numpy as np
import pytest

import gradio as gr
from gradio import FileData


class TestLabel:
    def test_component_functions(self):
        """
        Process, postprocess, deserialize
        """
        y = "happy"
        label_output = gr.Label()
        label = label_output.postprocess(y).model_dump()  # type: ignore
        assert label == {"label": "happy", "confidences": None}

        y = {3: 0.7, 1: 0.2, 0: 0.1}
        label = label_output.postprocess(y).model_dump()  # type: ignore
        assert label == {
            "label": 3,
            "confidences": [
                {"label": 3, "confidence": 0.7},
                {"label": 1, "confidence": 0.2},
                {"label": 0, "confidence": 0.1},
            ],
        }
        label_output = gr.Label(num_top_classes=2)
        label = label_output.postprocess(y).model_dump()  # type: ignore

        assert label == {
            "label": 3,
            "confidences": [
                {"label": 3, "confidence": 0.7},
                {"label": 1, "confidence": 0.2},
            ],
        }
        with pytest.raises(ValueError):
            label_output.postprocess([1, 2, 3]).model_dump()  # type: ignore

        test_file_dir = Path(__file__).parent.parent / "test_files"
        path = str(test_file_dir / "test_label_json.json")
        label_dict = label_output.postprocess(path).model_dump()  # type: ignore
        assert label_dict["label"] == "web site"

        assert label_output.get_config() == {
            "name": "label",
            "show_label": True,
            "num_top_classes": 2,
            "value": {},
            "label": None,
            "container": True,
            "min_width": 160,
            "scale": None,
            "elem_id": None,
            "elem_classes": [],
            "visible": True,
            "proxy_url": None,
            "color": None,
            "_selectable": False,
            "key": None,
            "preserved_by_key": ["value"],
            "show_heading": True,
            "buttons": [],
        }

    def test_color_argument(self):
        label = gr.Label(value=-10, color="red")
        assert label.get_config()["color"] == "red"

    def test_in_interface(self):
        """
        Interface, process
        """
        x_img = FileData(path="test/test_files/bus.png")

        def rgb_distribution(img):
            rgb_dist = np.mean(img, axis=(0, 1))
            rgb_dist /= np.sum(rgb_dist)
            rgb_dist = np.round(rgb_dist, decimals=2)
            return {
                "red": rgb_dist[0],
                "green": rgb_dist[1],
                "blue": rgb_dist[2],
            }

        iface = gr.Interface(rgb_distribution, "image", "label")
        output = iface(x_img)
        assert output == {
            "label": "red",
            "confidences": [
                {"label": "red", "confidence": 0.44},
                {"label": "green", "confidence": 0.28},
                {"label": "blue", "confidence": 0.28},
            ],
        }



import gradio as gr


class TestMarkdown:
    def test_component_functions(self):
        markdown_component = gr.Markdown("# Let's learn about $x$", label="Markdown")
        assert markdown_component.get_config()["value"] == "# Let's learn about $x$"
        assert not markdown_component.get_config()["buttons"]

    def test_in_interface(self):
        """
        Interface, process
        """
        iface = gr.Interface(lambda x: x, "text", "markdown")
        input_data = "    Here's an [image](https://gradio.app/images/gradio_logo.png)"
        output_data = iface(input_data)
        assert output_data == input_data.strip()

    def test_show_copy_button(self):
        markdown_component = gr.Markdown("# Let's learn about $x$", buttons=["copy"])
        assert markdown_component.get_config()["buttons"] == ["copy"]



import inspect
import pathlib
from contextlib import contextmanager

import pytest
from gradio_client import Client

import gradio as gr
import gradio.utils


def pytest_configure(config):
    config.addinivalue_line(
        "markers", "flaky: mark test as flaky. Failure will not cause te"
    )


@pytest.fixture
def test_file_dir():
    return pathlib.Path(pathlib.Path(__file__).parent, "test_files")


@pytest.fixture
def io_components():
    all_subclasses = gradio.utils.core_gradio_components()
    subclasses = []
    for subclass in all_subclasses:
        if subclass in [
            gr.components.FormComponent,
            gr.State,
            gr.LoginButton,
            gr.Timer,
            gr.Dialogue,
            gr.Navbar,
        ]:
            continue
        if subclass in gr.components.FormComponent.__subclasses__():
            continue

        if "value" in inspect.signature(subclass.__init__).parameters:
            subclasses.append(subclass)

    return subclasses


@pytest.fixture
def connect():
    @contextmanager
    def _connect(demo: gr.Blocks, **kwargs):
        _, local_url, _ = demo.launch(prevent_thread_lock=True, **kwargs)
        try:
            client = Client(local_url)
            yield client
        finally:
            client.close()  # type: ignore
            demo.close()

    return _connect


@pytest.fixture(autouse=True)
def gradio_temp_dir(monkeypatch, tmp_path):
    """tmp_path is unique to each test function.
    It will be cleared automatically according to pytest docs: https://docs.pytest.org/en/6.2.x/reference.html#tmp-path
    """
    monkeypatch.setenv("GRADIO_TEMP_DIR", str(tmp_path))
    return tmp_path


@pytest.fixture(autouse=True)
def clear_static_files():
    """Clears all static files from the _StaticFiles class.

    This is necessary because the tests should be independent of one another.
    """
    yield
    from gradio import data_classes

    data_classes._StaticFiles.clear()


@pytest.fixture
def test_mcp_app():
    """Fixture that creates a test Gradio app with a simple text processing tool for MCP tests."""

    def test_tool(x: str) -> str:
        """
        This is a test tool.
        Parameters:
        - x: str
        Returns:
        - the original value as a string
        """
        return str(x)

    with gr.Blocks() as app:
        t1 = gr.Textbox(label="Test Textbox")
        t2 = gr.Textbox(label="Test Textbox 2")
        t1.submit(test_tool, t1, t2)

    return app


@pytest.fixture
def media_data():
    import sys
    from pathlib import Path

    sys.path.append(Path("..", "client", "python", "test").resolve().as_posix())
    from client.python.test import media_data

    return media_data


@pytest.fixture
def stateful_mcp_app():
    def process(name: str, hidden_state: str, flag: bool, gallery_images):
        """
        Process inputs and return a result.

        Args:
        name: A text input from user
        hidden_state: A value from gr.State (hidden from MCP)
        flag: A boolean checkbox
        gallery_images: Gallery images list

        Returns:
        str: Result showing all received values
        """
        return f"name={name}, hidden_state={hidden_state}, flag={flag}, gallery={gallery_images}"

    with gr.Blocks() as demo:
        gr.Markdown("# Gradio MCP Bug: gr.State breaks parameter order")

        name_input = gr.Textbox(label="Name", value="test")
        hidden_state = gr.State(value="hidden_value")
        flag_input = gr.Checkbox(label="Flag", value=True)
        gallery = gr.Number(label="Images")

        output = gr.Textbox(label="Result")

        btn = gr.Button("Process")
        btn_nq = gr.Button("Process No Queue")
        btn.click(
            process,
            inputs=[name_input, hidden_state, flag_input, gallery],
            outputs=[output],
            api_visibility="public",
        )
        btn_nq.click(
            process,
            inputs=[name_input, hidden_state, flag_input, gallery],
            outputs=[output],
            api_visibility="public",
            api_name="no_queue",
            queue=False,
        )
    return demo
